/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */

#ifndef INCL_CONFIG_PATHS_HH
#define INCL_CONFIG_PATHS_HH

#include <glibmm/ustring.h>

// A word on public const data members:
// Some people feel it's a mortal sin of OOP to make any data member public
// under any circumstances. My feeling is that it's a safe thing to do when
// we want the data to be globally read-only throughout the program's 
// execution and throughout the duration of the program's development 
// life-cycle. We ONLY want to assign values to these strings at 
// initialization time, we want the initialization itself to be independent 
// of client code, and we NEVER want the case to be otherwise during the 
// duration of Gabber2 development. So having an accessor for each data 
// member of each class in this file would be superfluous. 
//
// But hey, I could be wrong -- we're experimenting a bit with Gabber2, and
// I'll be the guy to blame later if this turns out to be a Bad Idea(TM). ;)
//              -- James

namespace Gabber {

namespace Keys {

class Account {
public:
    const Glib::ustring main_dir;     // Where Account stuff is stored
    const Glib::ustring savepassword; // Holds a bool
    const Glib::ustring password;     // Holds a Glib::ustring
    const Glib::ustring port;         // Holds a gint
    const Glib::ustring priority;     // Holds a gint
    const Glib::ustring resource;     // Holds a Glib::ustring
    const Glib::ustring server;       // Holds a Glib::ustring
    const Glib::ustring username;     // Holds a Glib::ustring

    Account() :
    main_dir     ("/apps/gabber/account"),
    savepassword ( main_dir + "/savepassword"),
    password     ( main_dir + "/password"),
    port         ( main_dir + "/port"),
    priority     ( main_dir + "/priority"),
    resource     ( main_dir + "/resource"),
    server       ( main_dir + "/server"),
    username     ( main_dir + "/username")
    {}
};
static const Account acct;
  
class Plugin {
public:
    const Glib::ustring main_dir;
    const Glib::ustring pluginlist;

    Plugin() : 
    main_dir     ("/apps/gabber/plugins"),
    pluginlist   ( main_dir + "/pluginlist")
    {}
};
static const Plugin plug;

}// ...More config path categories to come....

}; // namespace Gabber
#endif // INCL_CONFIG_PATHS_HH
