/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */
#ifndef INCL_GABBER_APP_HH
#define INCL_GABBER_APP_HH

#include <libintl.h>
#define _(String) gettext (String)

#include "Singleton.hh"
#include "JabberConnection.hh"
#include "ConfigWrapper.hh"
#include "TCPTransmitter.hh"
#include "GabberWin.hh"
#include "ResourceManager.hh"
#include "LogManager.hh"
#include <jabberoo/jabberoo.hh>

#include <gtkmm/main.h>
#include <sigc++/object.h>


/**
 * Primary Gabber Namespace
 */
namespace Gabber
{

class GabberWin;

/**
 * The main Gabber application object.
 * This class is instantiated by main and is
 * in control of the Gtk::Main loop.
 * It is also the direct control over TCPTransmitter
 * and jabberoo::Session. It therefore has functions
 * such as login and logout which are preferred to
 * accessing jabberoo::Session and TCPTransmitter 
 * directly.
 */
class GabberApp
     : public Singleton<GabberApp>, public JabberConnection
{
public:
     /**
      * The constructor intended for main
      */
     GabberApp(int argc, char** argv);
     ~GabberApp();

     // Modifiers
     /**
      * Start the Gtk::Main loop.
      * This should should only be started by main.
      */
     void run();
     /**
      * Initialize the main Gabber Window.
      * This will make G_Win no longer null.
      */
     void init_win();
     /**
      * Cleanly disconnect, sending offline presence and all.
      */
     void clean_disconnect();
     /**
      * Quit the application, kill the Gtk::Main loop, etc.
      */
     void quit();

     /**
     * Get a reference to the resource manager
     */
     ResourceManager& getResourceManager() const
     { return *_resource_manager; }

     /**
     * Return a reference to the agent list
     */
     jabberoo::DiscoDB::JIDList& getAgentList()
     { return _agents; }

     /**
     * This signal is fired whenever we need to display a groupchat join dialog
     * There are issues with how this is implemented right now, such as multiple
     * handlers could hook up to it, but we really only want one to do so.
     */
     SigC::Signal0<void> sigJoinGCDlg;

protected:
     // Session event handlers
     void on_XML_transmit(const char* XML);
     void on_XML_recv(const char* XML);
     void on_presence_request(const jabberoo::Presence& pres);

private:
    Gtk::Main           _GtkMain;
    GabberWin*          _winMain;
    ResourceManager*    _resource_manager;
    jabberoo::DiscoDB::JIDList    _agents;
    jabberoo::DiscoDB::DiscoFilter* _agent_filter;
    LogManager*         _log_manager;

    struct AgentFilter : jabberoo::DiscoDB::FilterPred
    {
    public:
        bool operator()(const jabberoo::DiscoDB::Item& item)
        {
            const jabberoo::DiscoDB::Item::FeatureList& features = item.getFeatureList();
            if (find(features.begin(), features.end(), "jabber:iq:gateway") !=
                    features.end())
            {
                return true;
            }
            return false;
        }
    };
};

/**
* Easier access for all you pansy typists
*/
#define G_App GabberApp::getSingleton()

}; // namespace Gabber

#endif // INCL_GABBER_APP_HH
