/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */

#include "config.h"

#include "GabberWin.hh"

#include "GabberApp.hh"
#include "RosterView.hh"
#include "RosterModel.hh"
#include "PrefsInterface.hh"

#include "AddContactDlg.hh"
#include "AgentsWindow.hh"

#include <gtkmm/dialog.h>
#include <gtkmm/textview.h>

using namespace jabberoo;

namespace Gabber {

GabberWin::GabberWin(GabberApp& app)
     : BaseGabberWindow("Gabber_win"), _app(app), _prev_show(1)
{
//     _thisWindow->realize();
     
     // Setup roster
     _rostermodel = new RosterModel(_app.getSession());
     Gtk::TreeView* tv;
     get_widget("Roster_treeview", tv);
     _rosterview  = new RosterView(_app, tv, _rostermodel, _thisWindow);
     
     // Initialize pointers
     get_widget("Presence_optmenu", _optPres);
     
     // Status area
     get_widget("StatusMsg_lbl", _lblStatus);
     get_widget("StatusMsg_scroll", _scrollStatus);
     get_widget("StatusMsg_txt", _txtStatus);

     _txtStatus->signal_event().connect(slot(*this, &GabberWin::on_status_txt_event));
     _txtStatus->signal_focus_out_event().connect(slot(*this, &GabberWin::on_status_txt_unfocused));


     // Presence events
     _app.getSession().evtMyPresence.connect(SigC::slot(*this, &GabberWin::on_my_presence_event));
     
     _app.evtConnecting.connect(  SigC::slot(*this, &GabberWin::on_evtConnecting));
     _app.evtConnected.connect(   SigC::slot(*this, &GabberWin::on_evtConnected));
     _app.evtDisconnected.connect(SigC::slot(*this, &GabberWin::on_evtDisconnected));

     // Hook up all the menus
     init_menus();
     
     show();
}

GabberWin::~GabberWin()
{
     delete _rosterview;
}

void GabberWin::init_menus() {
    Gtk::MenuItem* mi;
    // Gabber
    get_widget("Quit_item", mi);
    mi->signal_activate().connect(SigC::slot(*this, &GabberWin::close));
    get_widget("Login_item", mi);
    mi->set_sensitive(true);
    mi->signal_activate().connect(SigC::slot(&LoginDlg::execute));
    get_widget("Logout_item", mi);
    mi->set_sensitive(false);
    mi->signal_activate().connect(SigC::slot(*this, &GabberWin::on_logout_item_activate));
    // Actions
//    get_widget("Message_item", mi);
//    mi->signal_activate().connect(SigC::slot(*this, &GabberWin::on_message_activate));
    get_widget("AddContact_item", mi);
    mi->signal_activate().connect(SigC::slot(*this, &GabberWin::on_add_contacts_activate));
    get_widget("BrowseAgents_item", mi);
    mi->signal_activate().connect(SigC::slot(*this, &GabberWin::on_browse_agents_activate));
    get_widget("JoinGroupChat_item", mi);
    mi->signal_activate().connect(SigC::slot(*this,
        &GabberWin::on_joingc_activate));
    // Roster Stuff
    get_widget("OfflineContacts_item", mi);
    mi->signal_activate().connect(SigC::slot(*this,
        &GabberWin::on_roster_hideoffline_activate));

    // Presence option menu
    _optPres->set_sensitive(false);
    _optPres->get_menu()->set_sensitive(false);
    _optPres->set_history(1);
    _optPres->signal_changed().connect(slot(*this,
                &GabberWin::on_pres_menu_changed));
}

void GabberWin::close() 
{
     _app.quit();
     BaseGabberWindow::close();
}

void GabberWin::on_evtConnecting()
{
     Gtk::MenuItem* mi;
     get_widget("Login_item", mi);
     mi->set_sensitive(false); // don't let the user try to log in again.
     get_widget("Logout_item", mi);
     mi->set_sensitive(true); // allow the user the option to log out.
}

void GabberWin::on_evtConnected()
{
     // We can now set presence. Yey.
     _optPres->set_sensitive(true);
     _optPres->get_menu()->set_sensitive(true);

     _app.getSession() << Presence("", Presence::ptAvailable,
				   Presence::stOnline, _("Online"));

     
    // And now.. the HACKS
    // This is one of the oldest HACKS in Gabber's existence
    // and yet, Jabber still has yet to provide a nice way around this.
    // How do we identify which roster items are Gateways without browsing to everyone?
    // ...
    // ...
    // That's right--we guess. If it doesn't have an @, then hey, maybe it's a Gateway.
    // this hack brought to you by Julian Missig, maker of fine Jabber hacks since 0.7.

    // Iterate over the whole roster
    for (Roster::iterator it = _app.getSession().roster().begin();
         it != _app.getSession().roster().end(); ++it)
    {
        // there's no @
        if(it->getJID().find("@") == string::npos)
        {
            // so maybe we have a Gateway?
            // this time around we can at least check..
            _app.getSession().discoDB().cache(it->getJID(), 
                slot(*this, &GabberWin::on_discoDB_cache));
        }
    }
}

void GabberWin::on_discoDB_cache(const jabberoo::DiscoDB::Item* item)
{
     // XXX: This doesn't seem to get called when it falls back on browse?
     // I don't think we particularly need to do anything here yet, but still...
}

void GabberWin::on_evtDisconnected()
{
     Gtk::MenuItem* mi;
     get_widget("Login_item", mi);
     mi->set_sensitive(true); // let the user try to log in again.
     get_widget("Logout_item", mi);
     mi->set_sensitive(false); // don't allow the user the option to log out.

     // We can't set presence anymore
     _optPres->set_sensitive(false);
     _optPres->get_menu()->set_sensitive(false);

     // We don't display a roster anymore
     _rostermodel->clear();
}

void GabberWin::on_my_presence_event(const Presence& p)
{
     // Set the presence menu
     switch(p.getShow())
     {
     case Presence::stChat:
	  _optPres->set_history(0);
	  break;
     case Presence::stAway:
	  _optPres->set_history(3);
	  break;
     case Presence::stXA:
	  _optPres->set_history(4);
	  break;
     case Presence::stDND:
	  _optPres->set_history(2);
	  break;
     default:
	  _optPres->set_history(1);
	  break;
     };
     
     // Set the presence status label
     _lblStatus->set_label(p.getStatus());
}

void GabberWin::on_logout_item_activate()
{
     _app.clean_disconnect();
}

void GabberWin::on_add_contacts_activate()
{
     // shake that thing
     AddContactDlg::display(_app, *_thisWindow);
}

void GabberWin::on_browse_agents_activate()
{
    // do it
    AgentsWindow* awin = new AgentsWindow(_app);
    awin->show();
}

void GabberWin::on_joingc_activate()
{
    // We actually pass this off to the modules
    _app.sigJoinGCDlg();
}

void GabberWin::on_roster_hideoffline_activate()
{
    Gtk::CheckMenuItem* mi;
    get_widget("OfflineContacts_item", mi);
    _rostermodel->setHideOffline(mi->get_active());
}


void GabberWin::on_pres_menu_changed()
{
    // Don't show the text view if they didn't actually change anything
    if (_prev_show == _optPres->get_history())
        return;
    
    // Hide the status label
    _lblStatus->hide();

    // Reset the text view and show it
    _txtStatus->get_buffer()->set_text(Glib::ustring());
    _txtStatus->show();
    _scrollStatus->show();

    // Focus on the text view
    _txtStatus->grab_focus();
}

bool GabberWin::on_status_txt_event(GdkEvent* ev) 
{
    if (ev->type != GDK_KEY_PRESS)
    return false;

    GdkEventKey* e = (GdkEventKey*)ev;


    // If they pressed the Keypad enter, make it act like a normal enter
    if (e->keyval == GDK_KP_Enter)
        e->keyval = GDK_Return;

    if (e->keyval == GDK_Return)
    {
        //enter a newline if shift-return is used
        if (e->state & GDK_SHIFT_MASK)
        {
            //unset the shift bit. shift-return seems to have a special meaning for the widget
            e->state ^= GDK_SHIFT_MASK;
            return false;
        }

        // Ctrl-Return sets status so we don't need to check for it

        set_status_from_status_txt();
        return true;
    }

    return false;
}

bool GabberWin::on_status_txt_unfocused(GdkEventFocus* e)
{
     if (_txtStatus->is_visible())
	  set_status_from_status_txt();

     return false;
}

void GabberWin::set_status_from_status_txt()
{
     Glib::RefPtr<Gtk::TextBuffer> tb = _txtStatus->get_buffer();
     Glib::ustring statusmsg = tb->get_text(tb->begin(), tb->end());

     // Set the status label
     _lblStatus->set_label(statusmsg);

     // The status label height multiplier
     int multiplier = 1;

     // Send the appropriate presence
     switch(_optPres->get_history())
     {
     case 1:
	  _app.getSession() << Presence("", Presence::ptAvailable,
					Presence::stOnline, statusmsg);
	  multiplier = 1;
	  break;
     case 0:
	  _app.getSession() << Presence("", Presence::ptAvailable,
					Presence::stChat, statusmsg);
	  multiplier = 1;
	  break;
     case 3:
	  _app.getSession() << Presence("", Presence::ptAvailable,
					Presence::stAway, statusmsg);
	  multiplier = 5;
	  break;
     case 4:
	  _app.getSession() << Presence("", Presence::ptAvailable,
					Presence::stXA, statusmsg);
	  multiplier = 6;
	  break;
     case 2:
	  _app.getSession() << Presence("", Presence::ptAvailable,
					Presence::stDND, statusmsg);
	  multiplier = 4;
	  break;
     case 5:
	  _app.getSession() << Presence("", Presence::ptInvisible,
					Presence::stOnline, statusmsg);
	  multiplier = 3;
	  break;
     };

     // Save the show selection
     _prev_show = _optPres->get_history();

     // Hide the text view
     _scrollStatus->hide();

     // Figure out how tall the status label should be
     
     // Figure out the line height
     // THEME/FONT SPECIFIC
     Glib::RefPtr<Pango::Layout> playout = Pango::Layout::create(_lblStatus->get_pango_context());
     Pango::Rectangle inkrect, logicalrect;
     playout->get_pixel_extents(inkrect, logicalrect);

     // Set the label height to be line height times multiplier
     _lblStatus->set_size_request(-1, logicalrect.get_height() * multiplier);

     // Show the status label
     _lblStatus->show();
}

} // namespace Gabber
