/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */

#ifndef __GABBER_INCL_TEXTVIEW_HH__
#define __GABBER_INCL_TEXTVIEW_HH__

#include "PrettyText.hh"

#include <gtkmm/textview.h>
#include <gtkmm/scrolledwindow.h>

namespace Gabber
{

/**
 * A PrettyText implementation using regular Gtk::TextView
 */
class PlainTextView : public PrettyText, public SigC::Object
{
public:
    /**
     * Create a new PlainTextView
     * @param scr_win The scrolled window to interact with.
     */
    PlainTextView(Gtk::ScrolledWindow* scr_win);
    ~PlainTextView();

    void append(const jabberoo::Message& m, const Glib::ustring& nick, 
                bool local_msg = false);
    void append(const Glib::ustring& str);
    void append_timestamp();
    void composing(const std::string& jid, const Glib::ustring& nick);
    void cancel_composing(const std::string& jid);

    Gtk::Widget* get_widget()
    {
        return _view;
    }

private:
    Glib::RefPtr<Gtk::TextBuffer> _buffer;
    Gtk::TextView* _view;
    Gtk::ScrolledWindow* _scr_win;

    typedef Glib::RefPtr<Gtk::TextMark> txt_mark_ref;
    typedef std::set<std::string> mark_map;
    mark_map _composers;

    bool   _goto_bottom; // Should _scr_win automatically scroll to its
                         // bottom when text is appended to it? 
    txt_mark_ref _bottom;// The bottom of the TextView
    txt_mark_ref _start_of_last_msg; // The name says it all.

    int _view_height;    // We need to store the size of the TextView
    int _view_width;     // so that we'll know when it changes, and therefore
                         // when to auto-scroll down.
};

}; //namespace Gabber

#endif // __GABBER_INCL_TEXTVIEW_HH__
