/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */

#include "PrefsInterface.hh"

#include "ConfigWrapper.hh"
#include "GabberApp.hh"

namespace Gabber {

// ---------------------------------------------------------
//
// Login Dialog
//
// ---------------------------------------------------------

LoginDlg* LoginDlg::_Dialog = NULL;

void LoginDlg::execute()
{
     // Create a login dialog
     _Dialog = manage(new LoginDlg(false));
}

LoginDlg::LoginDlg(bool is_settings)
     : BaseGabberWindow("Login_dlg"), ConfigWrapper(Keys::acct.main_dir), _is_settings(is_settings)
{
     // Connect buttons to handlers
     Gtk::Button* b = 0;
     get_widget("Cancel_btn", b);
     b->signal_clicked().connect(SigC::slot(*this, &LoginDlg::on_Cancel_clicked));
     get_widget("LogIn_btn", _btnLogIn);
     _btnLogIn->signal_clicked().connect(SigC::slot(*this, &LoginDlg::on_LogIn_clicked));

     // Initialize pointers
     get_widget("Username_ent", _entUsername);
     _entUsername ->signal_changed().connect(SigC::slot(*this, &LoginDlg::on_changed));
     get_widget("Server_ent", _entServer);
     _entServer   ->signal_changed().connect(SigC::slot(*this, &LoginDlg::on_changed));
     get_widget("Password_ent", _entPassword);
     _entPassword ->signal_changed().connect(SigC::slot(*this, &LoginDlg::on_changed));
     get_widget("Resource_ent", _entResource);
     _entResource ->signal_changed().connect(SigC::slot(*this, &LoginDlg::on_changed));
     get_widget("Port_spin", _spinPort);
     _spinPort    ->set_width_chars(4);
     _spinPort    ->signal_changed().connect(SigC::slot(*this, &LoginDlg::on_changed));
     get_widget("Priority_spin", _spinPriority);
     _spinPriority->signal_changed().connect(SigC::slot(*this, &LoginDlg::on_changed));
     get_widget("SavePassword_chk", _ckPassword);
     _ckPassword->signal_toggled().connect(SigC::slot(*this, &LoginDlg::on_changed));

     // Load configuration
     loadconfig();
    
     // display
     show();
}

LoginDlg::~LoginDlg()
{
     // Initialize the main window
     G_App.init_win();
}

void LoginDlg::loadconfig()
{
    _entUsername-> set_text  (client->get_string(Keys::acct.username));
    _entServer->   set_text  (client->get_string(Keys::acct.server));
    _entResource-> set_text  (client->get_string(Keys::acct.resource));
    _entPassword-> set_text  (client->get_string(Keys::acct.password));
    _spinPort->    set_value (client->get_int   (Keys::acct.port));
    _spinPriority->set_value (client->get_int   (Keys::acct.priority));
    _ckPassword->  set_active(client->get_bool  (Keys::acct.savepassword));
}

void LoginDlg::saveconfig()
{
    client->set(Keys::acct.username    ,_entUsername-> get_text());
    client->set(Keys::acct.server      ,_entServer->   get_text());
    client->set(Keys::acct.resource    ,_entResource-> get_text());
    client->set(Keys::acct.password    ,_entPassword-> get_text());
    client->set(Keys::acct.port        ,_spinPort->    get_value_as_int());
    client->set(Keys::acct.priority    ,_spinPriority->get_value_as_int());
    client->set(Keys::acct.savepassword,_ckPassword->  get_active());
}

void LoginDlg::on_LogIn_clicked()
{
     // Save configuration
     saveconfig();

     // Initialize the main window
     G_App.init_win();

     // Start the login process
     G_App.connect();

     // And now close it all up, we're done
     close();
}

void LoginDlg::on_Cancel_clicked()
{
     close();
}

void LoginDlg::on_changed()
{
     // If they pass our test, let them click Log In
     if (jabberoo::JID::isValidUser(_entUsername->get_text()) &&
	 jabberoo::JID::isValidHost(_entServer->get_text()) &&
	 !_entPassword->get_text().empty() &&
	 !_entResource->get_text().empty())
     {
	  _btnLogIn->set_sensitive(true);
     }
     else
     {
	  _btnLogIn->set_sensitive(false);
     }
}

// gint LoginDlg::on_Username_key_press(GdkEventKey* e)
// {
//      if (e->keyval == GDK_at)
//      {
// 	  // HACK --  to stop the signal from continuing, and the @ from being put into the Username field
// 	  gtk_signal_emit_stop_by_name(GTK_OBJECT(_entUsername->gtkobj()), "key_press_event");
// 	  _entServer->grab_focus();
//      }
//      return 0;
// }

} // namespace Gabber
